@@ -15,13 +15,14 @@ padModel.syn <- function(data, method, predictor.matrix, visit.sequence,
  
  # This is a data frame with a row for each variable and extra rows for 
  # each of the dummy variables added at the end of the j loop with
- # col1 = T/F if category and predictor in parametric model                     #!BN1605
- # col2 = number of dummy variables for factors with col1==TRUE, else 0         #!BN1605
+ # col1 = T/F if category and predictor in parametric model                   #!BN1605
+ # col2 = number of dummy variables for factors with col1==TRUE, else 0       #!BN1605
  # col3 = yes.no.dummy TRUE for dummy variables
  # col4 = corresponding.column.dummy original column number for dummies, else 0
 
-  pred.with.cart <- method %in% c("ctree","ctree.proper","cart","cart.proper","myctree","mycart","collinear")  #!BN-1605; "myctree","mycart" - GR's test functions 
-                                                                                                 #!BN to check if collinear needed
+  pred.with.cart <- method %in% c("ctree", "ctree.proper", "cart", "cart.proper", 
+                                  "myctree", "mycart", "collinear", "satcat")  #!"myctree","mycart" - GR's test functions 
+                                                                               #!BN to check if collinear needed
   for(j in 1:nvar){
     if ((is.factor(data[,j]) & any(predictor.matrix[1:nvar,j]!=0 & !pred.with.cart)) |  #!BN-16/05/2016
         (factorNA[j]==TRUE & !pred.with.cart[j])){                                     #!BN-16/05/2016
@@ -37,13 +38,13 @@ padModel.syn <- function(data, method, predictor.matrix, visit.sequence,
       predictor.matrix <- rbind(predictor.matrix, matrix(0,
                                ncol=ncol(predictor.matrix), nrow=n.dummy))
       
-      # columns are set to zero and then for vars with non-CART method          #!BN1605
-      # copied from an original variable j in predictor.matrix for               #!BN1605
-      # -> 1 for all the rows for which this variable is being used as          #!BN1605
-      # a predictor in a non-CART model, 0 otherwise                            #!BN1605
-      predictor.matrix <- cbind(predictor.matrix, matrix(0, ncol=n.dummy,         #!BN1605
-                               nrow=nrow(predictor.matrix)))                     #!BN1605
-      predictor.matrix[!pred.with.cart,(ncol(predictor.matrix)-n.dummy+1):        #!BN1605
+      # columns are set to zero and then for vars with non-CART method        #!BN1605
+      # copied from an original variable j in predictor.matrix for            #!BN1605
+      # -> 1 for all the rows for which this variable is being used as        #!BN1605
+      # a predictor in a non-CART model, 0 otherwise                          #!BN1605
+      predictor.matrix <- cbind(predictor.matrix, matrix(0, ncol=n.dummy,     #!BN1605
+                               nrow=nrow(predictor.matrix)))                  #!BN1605
+      predictor.matrix[!pred.with.cart,(ncol(predictor.matrix)-n.dummy+1):    #!BN1605
         ncol(predictor.matrix)] <- matrix(rep(predictor.matrix[!pred.with.cart,j],times=n.dummy)) #!BN1605
                                
       # the original categorical variable is removed from predictors (=insert zeros)
